/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include "mpc8xx.h"

/* ------------------------------------------------------------------------- */

static long int dram_size (long int, long int *, long int);

/* ------------------------------------------------------------------------- */

#define	_NOT_USED_	0xFFFFFFFF

const uint sdram_table[] =
{
	/* Single read cycle */
0x1F07D004, 0xEEAEE004, 0x11ADD004, 0xEFBBA000,
0x1FF75447,
0x1FF77C34, /*Load mode register*/
0xEFEABC34, 0x1FB57C35, 
	/* Burst read */
0x1F07D004, 0xEEAEE004, 0x00ADC004, 0x00AFC000, 
0x00AFC000, 0x01AFC000, 0x0FBB8000, 0x1FF75447,
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
	/* Single write cycle */
0x1F27D004, 0xEEAEA000, 0x01B90004, 0x1FF75447,
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
	/* Burst Write cycle */
0x1F07D004, 0xEEAEA000, 0x00AD4000, 0x00AFC000,
0x00AFC000, 0x01BB8004, 0x1FF75447, _NOT_USED_, 
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
	/* reFresh cycle */
0x1FF5DC84, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04,
0xFFFFFC84, 0xFFFFFC07, _NOT_USED_, _NOT_USED_, 
_NOT_USED_, _NOT_USED_, _NOT_USED_, _NOT_USED_, 
	/* Exception */
0x7FFFFC07, _NOT_USED_, _NOT_USED_, _NOT_USED_
};

/* ------------------------------------------------------------------------- */


int checkboard (void)
{
	puts("FlagaDM V3.0");
	return 3; //Version 3
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
	volatile immap_t     *immap  = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;
	long int size_b0;

    memctl->memc_or2 = CFG_OR2_PRELIM;
    memctl->memc_br2 = CFG_BR2_PRELIM;

	udelay(1000);
	upmconfig(UPMA, (uint *)sdram_table, sizeof(sdram_table)/sizeof(uint));

	memctl->memc_mptpr = MPTPR_PTP_DIV16;

	memctl->memc_mamr = CFG_MAMR_48_SDR_8; 
	memctl->memc_mcr = 0x80004130;  // a refresh cycle
	udelay(100);
	
	//Mode register programming
	memctl->memc_mamr = CFG_MAMR_48_SDR_4;

	memctl->memc_mar = 0x00000088; 
	memctl->memc_mcr = 0x80004105;

    memctl->memc_mamr |= MAMR_PTAE;	/* enable refresh */
	
	udelay(100);

//	size_b0 = dram_size (CFG_MAMR_48_SDR_4, (ulong *)SDRAM_BASE2_PRELIM, SDRAM_MAX_SIZE);
	size_b0 = 16*1024*1024;

	/*
	 * No bank 1
	 *
	 * invalidate bank
	 */
	memctl->memc_br3 = 0;

	return (size_b0);
}

/* ------------------------------------------------------------------------- */

/*
 * Check memory range for valid RAM. A simple memory test determines
 * the actually available RAM size between addresses `base' and
 * `base + maxsize'. Some (not all) hardware errors are detected:
 * - short between address lines
 * - short between data lines
 */

static long int dram_size (long int mamr_value, long int *base, long int maxsize)
{
    volatile immap_t     *immap  = (immap_t *)CFG_IMMR;
    volatile memctl8xx_t *memctl = &immap->im_memctl;
    volatile long int	 *addr;
    long int		  cnt, val;

    memctl->memc_mamr = mamr_value;

    for (cnt = maxsize/sizeof(long); cnt > 0; cnt >>= 1) {
	addr = base + cnt;	/* pointer arith! */

	*addr = ~cnt;
    }

    /* write 0 to base address */
    addr = base;
    *addr = 0;

    /* check at base address */
    if ((val = *addr) != 0) {
	return (0);
    }

    for (cnt = 1; ; cnt <<= 1) {
	addr = base + cnt;	/* pointer arith! */

	val = *addr;

	if (val != (~cnt)) {
	    return (cnt * sizeof(long));
	}
    }
    /* NOTREACHED */
}
